/**
 * Class Plotdata
 * 
 * Stores data for one 2D plotting axis
 *
 * Provides operators and mathematical functions to map a Plotdata
 * object to another using unary and binary functions.
 *
 * Provides insertion and extraction of special commands into and from
 * Plotdata pairs as static operations.
 * Available commands are:
 *      colourChange (to change the foreground colour of graphs)
 *      singlePoint  (to draw a single point of data as a special marker)
 *
 * Author: 	jlk
 * Version:	1.1
 * Date:	July 2005
 */
 
#pragma once

//#define _USE_MATH_DEFINES
#include <cmath>
#include <vector>
#include <iostream>
#include <limits>
#include <functional>
#include <algorithm>
#include <iterator>

// BEGIN Windows-specific, MSVC2008 specific
#include <windows.h>
#undef min
#undef max
#undef T
typedef COLORREF Color;	// central switch between GDI and GDIplus
#define COLOR(r,g,b) RGB(r,g,b)

const Color BLACK	=RGB( 0, 0, 0 );
const Color BLUE	=RGB( 0, 0, 128);        
const Color GREEN	=RGB( 0, 128, 0 );       
const Color CYAN	=RGB( 0, 128, 128 );     
const Color RED		=RGB( 128, 0, 0 );       
const Color MAGENTA	=RGB( 128, 0, 128 );     
const Color BROWN	=RGB( 128, 128, 0 );     
const Color LIGHTGRAY	=RGB( 192, 192, 192 );   
const Color DARKGRAY	=RGB( 128, 128, 128 );   
const Color LIGHTBLUE	=RGB( 128, 128, 255 );   
const Color LIGHTGREEN	=RGB( 128, 255, 128 );   
const Color LIGHTCYAN	=RGB( 128, 255, 255 );   
const Color LIGHTRED	=RGB( 255, 128, 128 );   
const Color LIGHTMAGENTA=RGB( 255, 128, 255 );   
const Color YELLOW	=RGB( 255, 255, 0 );     
const Color WHITE	=RGB( 255, 255, 255 );   

const Color CRIMSON		=COLOR(220, 20, 60);
const Color CHOCOLATE		=COLOR(210, 105, 30);
const Color GOLDENROD		=COLOR(218, 165, 32);
const Color DARKORANGE		=COLOR(255, 140, 0);
const Color REDRED		=COLOR(255, 0, 0);
const Color DARKSLATEGRAY	=COLOR(47, 79, 79);
const Color DEEPPINK		=COLOR(255, 20, 147);
const Color TURQUOISE		=COLOR(64, 224, 208);
const Color DARKVIOLET		=COLOR(148, 0, 211);
const Color BEIGE		=COLOR(245, 245, 220);
const Color MEDIUMSEAGREEN	=COLOR(60, 179, 113);
const Color LIMEGREEN		=COLOR(50, 205, 50);
const Color DARKGREEN		=COLOR(0, 100, 0);
const Color MEDIUMBLUE		=COLOR(0, 0, 205);
const Color BLUEBLUE		=COLOR(0, 0, 255);
const Color DODGERBLUE		=COLOR(30, 144, 255);

/* -------------------------------------------------------- */

//namespace std{
 template<class T> bool isfinite(T);
 template<class T> T round(T);
 template<class T> T remquo(T,T,int*);
 template<class T> T trunc(T);
//}
// END Windows-specific, MSVC2008 specific
//typedef double float_t;

template<class T> struct is_finite {
 bool operator() (T val) const {return isfinite(val);}
};

using namespace std;

// Unary function pointer type accepted by Plotdata (double parameter)
template<class T> struct Func{
 typedef T(*Fu)(T);
 Fu ptr;
 void operator=(Fu x) {ptr=x;}
 Func(Fu x=0):ptr(x) {}
 bool operator!() {return !ptr;}
};
// Binary function pointer type accepted by Plotdata
template<class T> struct BinFunc{
 typedef T(*Fu)(T,T);
 Fu ptr;
 void operator=(Fu x) {ptr=x;}
 BinFunc(Fu x=0):ptr(x) {}
};

/** When true defines a range to be logarithmic */
//typedef bool LogSpace;

/** Define data iterator */
//template<class T>typedef vector<float_t>::const_iterator  dataIterator;

template<class T>union Point{
 struct{T x,y;};
 T A[2];
 T&operator[](int i) {return A[i];}
 const T&operator[](int i)const{return A[i];}
};

/** Remove case sensitivity for Plotdata class */
template<class T>union Rect{
 struct{T l,t,r,b;};
 struct{Point<T> tl,br;};
 Point<T> p[2];
 T A[4];
 T&operator[](int i) {return A[i];}
 const T&operator[](int i)const{return A[i];}
 T w() const {return r-l;}
 T h() const {return b-t;}
};

template<class T>struct Range{
 T min,max;
 Range() {reset();}
 Range(const Range&r) {reset(r);}
 Range(T _min,T _max) {reset(_min,_max);}
 void reset() {reset(std::numeric_limits<T>::max(),std::numeric_limits<T>::min());}
 void reset(T v) {reset(v,v);}
 void reset(T _min,T _max) {min=_min; max=_max;}
 void reset(const Range&r) {reset(r.min,r.max);}
 Range&operator=(const Range&r) {reset(r); return*this;}
 bool expand(T v) {return expand(v,v);}
 bool expand(T _min, T _max) {return setMin(min,_min) & setMax(max,_max);}
 bool expand(const Range&r) {return expand(r.min,r.max);}
 bool operator+(T v) {return expand(v);}
 bool operator+(const Range&r) {return expand(r);}
 bool inside(T v) const {return inside(v,v);}
 bool inside(T _min,T _max) const {return min<=_min && _max<=max;}
 bool inside(const Range&r) const {return inside(r.min,r.max);}
 bool equal(T _min,T _max) const {return min==_min && _max==max;}
 bool equal(const Range&r) const {return equal(r.min,r.max);}
 bool operator==(const Range&r) const {return equal(r);}
 bool overlap(T _min,T _max) const {return min<=_max || _min<=max;}
 bool overlap(const Range&r) const {return overlap(r.min,r.max);}
 T getDelta() const {return max-min;}
 bool isValid() const {return getDelta()>=0;}
 bool isRegular() const {return getDelta()>0;}
};

/****************************************************
 * (Beliebige, aber schnell rechnende) Skalierungen *
 ****************************************************/

template<class TX,class TY>struct Scale{
 virtual TY scale(TX) const=0;
 virtual TX unscale(TY) const=0;
 virtual void setup(const TX*,const TY*,size_t) {}
 virtual bool isBijective()	{return true;}
};

template<class T>struct NoScale:public Scale<T,T>{
 virtual T scale(T x) const override {return x;}
 virtual T unscale(T y) const override {return y;}
};

template<class TX,class TY,class TF=float>struct LinScale:public Scale<TX,TY> {
 TF factor;
 TY offset;
 LinScale(TF f=1,TY o=0):factor(f),offset(o) {}
 virtual TY scale(TX x) const override {return TY(x*factor)+offset;}
 virtual TX unscale(TY y) const override {return TX((y-offset)/factor);}
 virtual void setup(const TX*x, const TY*y, size_t) override {
  TF dx=TF(x[1])-TF(x[0]);	// cast: avoid TX overflow e.g. for +32000 - -32000
  if (dx) factor=(TF(y[1])-TF(y[0]))/dx;
  offset=y[0]-TY(x[0]*factor);
 }
};

template<class TX,class TY,class TF=float>struct LogScale:public Scale<TX,TY> {
 TF factor;
 TY offset;
 LogScale(TF f=1,TY o=0):factor(f),offset(o) {}
 virtual TY scale(TX x) const override {return TY(std::log(x*factor))+offset;}
 virtual TX unscale(TY y) const override {return TX(std::exp(TF(y-offset))/factor);}
 virtual void setup(const TX*x, const TY*y, size_t) override {
  TF dx=TF(x[1])-TF(x[0]);
  TF dy=TF(y[1])-TF(y[0]);
  if (dx>0) factor=dy/std::log(dx);	// TODO: Falsch!?
  offset=y[0]-TY(std::log(x[0])*factor);
 }
};

//Plotdata;

/** #define Plotdata & as a type (avoid using confusing '&' symbols in C) */

/** Number of points in a plot */
enum Grain
{
	EXTRA_FINE = 901,
	FINE = 501,
	MEDIUM = 301,
	COARSE = 201,
	GROSS = 101
};

const double NOPLOT = numeric_limits<double>::quiet_NaN(); // Any point of this value will not be plotted

const int NOCOLOR      = -1;
//const int RESETCOLOR   = -2;

template<class T=double>class Plotdata{
 friend ostream&operator<<(ostream & out, const Plotdata&pd)
	{out<<endl<<pd.data.size()<<endl;copy(pd.data.begin(),pd.data.end(),ostream_iterator<T>(out," "));return out;}
 friend istream&operator>>(istream & in, Plotdata& pd)
	{int size; T val; in>>size;
	 for (int i = size; i > 0; i--) {
	  in>>val;
	  pd.data.push_back(val);
         }
         return in;
	}
 friend Plotdata operator+ (T op1, const Plotdata& pd)
	{vector<T>vec(pd.data.size());transform(pd.data.begin(),pd.data.end(),vec.begin(),bind2nd(plus<T>(),op1));return Plotdata(vec);}
 friend Plotdata operator- (T op1, const Plotdata& pd)
	{vector<T>vec(pd.data.size());transform(pd.data.begin(),pd.data.end(),vec.begin(),bind2nd(minus<T>(),op1));return Plotdata(vec);}
 friend Plotdata operator* (T op1, const Plotdata&pd)
	{vector<T>vec(pd.data.size());transform(pd.data.begin(),pd.data.end(),vec.begin(),bind2nd(multiplies<T>(),op1));return Plotdata(vec);}
 friend Plotdata operator/ (T op1, const Plotdata&pd)
	{vector<T>vec(pd.data.size());transform(pd.data.begin(),pd.data.end(),vec.begin(),bind2nd(divides<T>(),op1));return Plotdata(vec);}
public:
 Plotdata operator+(const Plotdata&v) const
 	{int tSize=data.size(),vSize = v.data.size();size_t len = tSize < vSize ? tSize : vSize;
	 vector<T>vec(len);transform(data.begin(),data.begin()+len,v.data.begin(),vec.begin(),plus<T>());return Plotdata(vec);}
 Plotdata operator+(T v) const
 	{vector<T>vec(data.size());transform(data.begin(),data.end(),vec.begin(),bind2nd(plus<T>(),v));return Plotdata(vec);}
 Plotdata operator-(const Plotdata&v) const
	{size_t len = size() > v.size() ? size() : v.size();
	 vector<T>vec(len);transform(data.begin(),data.begin()+len,v.data.begin(),vec.begin(),minus<T>());return Plotdata(vec);}
 Plotdata operator-(T v) const
	{vector<T>vec(data.size());transform(data.begin(),data.end(),vec.begin(),bind2nd(minus<T>(),v));return Plotdata(vec);}
 Plotdata operator*(const Plotdata&v) const
	{size_t len = size() > v.size() ? size() : v.size();
	 vector<T>vec(len);transform(data.begin(),data.begin()+len,v.data.begin(),vec.begin(),multiplies<T>());return Plotdata(vec);}
 Plotdata operator*(T v) const
	{vector<T>vec(data.size());transform(data.begin(),data.end(),vec.begin(),bind2nd(multiplies<T>(),v));return Plotdata(vec);}
 Plotdata operator/(const Plotdata&v) const
	{size_t len = size() > v.size() ? size() : v.size();
	 vector<T>vec(len);transform(data.begin(),data.begin()+len,v.data.begin(),vec.begin(),divides<T>());return Plotdata(vec);}
 Plotdata operator/(T v) const
	{vector<T>vec(data.size());transform(data.begin(),data.end(),vec.begin(),bind2nd(divides<T>(),v));return Plotdata(vec);}
 Plotdata operator^(T v) const
 	{vector<T>vec(data.size());transform(data.begin(),data.end(),vec.begin(),bind2nd(ptr_fun(powl),v));return Plotdata(vec);}
 Plotdata operator-() const
 	{vector<T>vec(data);transform(vec.begin(),vec.end(),vec.begin(),negate<T>());return Plotdata(vec);}
 Plotdata&operator<<(const Plotdata&v)
 	{data.insert(data.end(),v.data.begin(),v.data.end());return*this;}	// concatenate
 Plotdata&operator<<(T v)
 	{data.push_back(v);return*this;}	// push_back a double to the data
// Constructors
 inline Plotdata(): data(MEDIUM)
	{}
 Plotdata(T lo, T hi, Grain grain=MEDIUM)
 	{Range<T>r(lo,hi); plotRange(r,grain);}
 Plotdata(const Range<T>&r, Grain grain=MEDIUM)
 	{plotRange(r,grain);}
 Plotdata(const T*ary, int dataSize):data(ary, ary + dataSize)
 	{}
 Plotdata(size_t s):data(s)
 	{}
 inline Plotdata(vector<T> d):data(d)
	{}
// Member Functions
 void insert(const T*ary, int dataSize)
 	{copy(ary, ary + dataSize, back_inserter(data));}	// Append, rather than insert at the start
 inline size_t size() const
 	{return data.size();}
 inline void point(T p)
 	{data.push_back(p);}
 void plotRange(const Range<T>&r,size_t numpoints,bool isLog=false)
	{
	 int divisions = numpoints - 1;
	 T lo=r.min,hi=r.max,rangeSize,shift;
	 if (divisions < 2) divisions = 49; // Guarantees a smooth enough curve
	 if (lo > hi) swap(lo,hi);
	// If space is logarithmic do nothing if hi and lo are of opposite signs
	 if (isLog && lo < 0 && hi > 0) return;
	 rangeSize = hi-lo;
	// if hi and lo are too close to each other do nothing
	 if (rangeSize < numeric_limits<T>::epsilon()) return;
	 data.clear();
	 if (!isLog) {
	  data.push_back(lo);
		// distance between 2 points
	  T dist = rangeSize / divisions; 
	  for (int i = divisions; i > 1; i--) data.push_back(lo += dist);
	 }else{ // Logarithmic range
		// Neither lo nor hi may be 0. Move away if too close
		// Dirty kludge! Avoids crash but introduces inaccuracies.
	  if (fabs(lo) < numeric_limits<T>::epsilon()) {
	   shift = rangeSize / 1000;
	   if ( lo < 0 ) shift = - shift;
	   while(fabsl(lo) < numeric_limits<T>::epsilon()) lo += shift;
	  }
	  if (fabs(hi) < numeric_limits<T>::epsilon()) {
	   shift = - rangeSize / 1000;
	   if ( hi > 0 ) shift = - shift;
	   while (fabsl(hi) < numeric_limits<T>::epsilon()) hi += shift;
	  }
	  data.push_back(lo);
	  T mult = log(hi/lo) / divisions;
	  for (int i = 1; i < divisions; i++) data.push_back(lo * expl(i * mult));
	 }
	 data.push_back(hi);
	}
 T minY() const;
 T maxY() const;
 void clear()
	{data.clear();}
 const vector<T>& getData() const
	{return data;}
 Func<T>&userfunc()
	{return userFunction;}
 BinFunc<T>&userBinfunc()
	{return userBinFunction;}
 Plotdata<T> doFunc(Func<T>f) const
	{Plotdata<T>ret(size());transform(data.begin(),data.end(),ret.data.begin(),f.ptr);return ret;}	 
 Plotdata<T> doBinFunc(BinFunc<T>f,T op2) const
	{Plotdata<T>ret(size());transform(data.begin(),data.end(),ret.data.begin(),bind2nd(ptr_fun(f),op2));return ret;}
 Plotdata<T> doBinFunc(T op2) const
	{Plotdata<T>ret(size());transform(data.begin(),data.end(),ret.data.begin(),bind2nd(ptr_fun(userBinFunction.ptr),op2));return ret;}
 static void rangeXY(const Plotdata &x, const Plotdata &y,Range<T>&rx,Range<T>&ry)
	{
	 rx.reset(); ry.reset();
    // Only search if both vectors contain some values
	 if (x.data.size() && y.data.size()){
	  dataIterator xIter = x.data.begin();
	  dataIterator yIter = y.data.begin();
	  dataIterator xEnd  = x.data.end();
	  dataIterator yEnd  = y.data.end();
        
	  for (; xIter != xEnd && yIter != yEnd; xIter++, yIter++) {
	   if (*xIter==*xIter && *yIter==*yIter) {	// Wenn Ordinate NaN enthlt, dann Abszisse nicht einrechnen, und umgekehrt
	    rx.expand(*xIter);
	    ry.expand(*yIter);
	   }
	  }
	 }
	}
private:
 vector<T> data;
 Func<T> userFunction;		// Any user-designed unary function
 BinFunc<T> userBinFunction;	// Any user-designed binary function
};
